@@ -1,6 +1,6 @@
 /* modiSBML2.c
 
-   Copyright (c) 2007-2010. Free Software Foundation, Inc.
+   Copyright (c) 2007-2017. Free Software Foundation, Inc.
 
    This file is part of GNU MCSim.
 
@@ -17,14 +17,6 @@
    You should have received a copy of the GNU General Public License
    along with GNU MCSim; if not, see <http://www.gnu.org/licenses/>
 
-   -- Revisions -----
-     Logfile:  %F%
-    Revision:  %I%
-        Date:  %G%
-     Modtime:  %U%
-      Author:  @a
-   -- SCCS  ---------
-
    Handles input parsing of the SBML Model Definition Files and of the
    template model (if used).
    Requires libSBML to be installed.
