@@ -179,8 +179,9 @@ BOOL DefDepParm (PSTR szLex, PDOUBLE pdValue, HANDLE *phvar)
 
 BOOL GetInputArgs (PINPUTBUF pibIn, PIFN pifn, int nArgs)
 {
-  PSTRLEX rgszLex[nArgs];
-  int rgiTypes[nArgs], i;
+  PSTRLEX *rgszLex = malloc (nArgs * sizeof(PSTRLEX));
+  int *rgiTypes = malloc (nArgs * sizeof(int));
+  int  i;
   BOOL bReturn = FALSE;
 
   for (i = 0; i < nArgs; i++)
@@ -207,6 +208,9 @@ BOOL GetInputArgs (PINPUTBUF pibIn, PIFN pifn, int nArgs)
       ReportError (pibIn, RE_EXPECTED, "input-spec", NULL);
   } /* if */
 
+  free(rgiTypes);
+  free(rgszLex);
+
   return (bReturn);
 
 } /* GetInputArgs */
@@ -248,15 +252,17 @@ BOOL GetNNumbers (PINPUTBUF pibIn, PSTR szLex, int nNumbers, PDOUBLE rgd)
 BOOL GetNDoses (PINPUTBUF pibIn, PSTR szLex, PIFN pifn)
 {
   PSTRLEX *rgszLex;
+  PSTRLEX  szTmp;
   int *rgiTypes, iType;
-  int i, iDoseArg;
+  long i, j, iLB, iUB, iDoseArg;
   BOOL bOK = TRUE;
   BOOL bErr = FALSE; /* Return value flags error condition */
+  HVAR hvar;
 
-  if ((bErr = EGetPunct (pibIn, szLex, CH_LPAREN)))
+  if ((bErr = EGetPunct(pibIn, szLex, CH_LPAREN)))
     goto Exit_GetNDoses;
 
-  if ((bErr = ENextLex (pibIn, szLex, LX_INTEGER)))
+  if ((bErr = ENextLex(pibIn, szLex, LX_INTEGER)))
     goto Exit_GetNDoses;
 
   pifn->nDoses = atoi(szLex);
@@ -286,28 +292,105 @@ BOOL GetNDoses (PINPUTBUF pibIn, PSTR szLex, PIFN pifn)
        (HANDLE *) malloc (pifn->nDoses * sizeof(HANDLE))))
     ReportError (pibIn, RE_OUTOFMEM | RE_FATAL, "GetNDoses", NULL);
 
-  /* Try to get doses list: n Mag's, n T0's */
+  /* Read the leading comma */
+  if (!(bOK = GetPunct (pibIn, rgszLex[0], ','))) {
+    *(rgszLex[0] + 1)  = ',';
+    ReportError(pibIn, RE_EXPECTED | RE_FATAL, rgszLex[0], NULL);
+  }
 
-  for (i = 0; i < iDoseArg && bOK; i++) {
+  /* Try to get a list of n magnitudes */
+  for (i = 0; i < (iDoseArg / 2) && bOK; i++) {
 
     rgiTypes[i] = LX_INTEGER | LX_FLOAT | LX_IDENTIFIER;
 
-    if (!(bOK = GetOptPunct (pibIn, rgszLex[i], ','))) {
-      *(rgszLex[i] + 1)  = ',';
-      ReportError(pibIn, RE_EXPECTED | RE_FATAL, rgszLex[i], NULL);
-      break; /* Error: Stop getting args */
-    }
-
-    NextLex (pibIn, rgszLex[i], &iType);
+    /* read the magnitude, numeric or symbolic */
+    NextLex (pibIn, szLex, &iType);
     if (!(bOK &= (iType & rgiTypes[i]) > 0))
       ReportError(pibIn, RE_LEXEXPECTED | RE_FATAL, vrgszLexTypes[rgiTypes[i]],
-                  rgszLex[i]);
-  } /* for */
+                  szLex);
 
-  if ((bErr = EGetPunct (pibIn, szLex, CH_RPAREN)))
+      /* check if it is a scalar or an array and act accordingly */
+      iLB = iUB = -1;
+      if (GetPunct (pibIn, szTmp, '[')) /* array found, read bounds */
+        GetArrayBounds (pibIn, &iLB, &iUB);
+
+      if (iUB == -1) { /* scalar, copy to rgszLex and continue */
+        strcpy (rgszLex[i], szLex);
+      }
+      else { /* array */
+
+        if (2 * (iUB - iLB) != iDoseArg)
+          ReportError (pibIn, RE_TOOMANYPVARS | RE_FATAL, "GetNDoses", NULL);
+
+        for (j = iLB; j < iUB; j++) {
+          sprintf (szTmp, "%s_%ld", szLex, j); /* create names */          
+
+          if ((bErr = !(hvar = GetVarHandle (szTmp))))
+            ReportError (pibIn, RE_UNDEFINED | RE_FATAL, szTmp, NULL);
+          else {
+            strcpy (rgszLex[i+j-iLB], szTmp);
+          }
+        } /* for j */
+
+        /* get the separating comma */
+       if ((bErr = !GetPunct (pibIn, szTmp, ',')))
+         goto Exit_GetNDoses;
+
+        break; /* get out of the for i loop */
+
+      } /* end else */
+
+  } /* for i */
+
+  /* Try to get a list of n T0's, numeric or symbolic */
+  for (i = iDoseArg / 2; i < iDoseArg && bOK; i++) {
+
+    rgiTypes[i] = LX_INTEGER | LX_FLOAT | LX_IDENTIFIER;
+
+    /* read the T0's, numeric or symbolic */
+    NextLex (pibIn, szLex, &iType);
+    if (!(bOK &= (iType & rgiTypes[i]) > 0))
+      ReportError(pibIn, RE_LEXEXPECTED | RE_FATAL, vrgszLexTypes[rgiTypes[i]],
+                  szLex);
+
+      /* check if it is a scalar or an array and act accordingly */
+      iLB = iUB = -1;
+      if (GetPunct (pibIn, szTmp, '[')) /* array found, read bounds */
+        GetArrayBounds (pibIn, &iLB, &iUB);
+
+      if (iUB == -1) { /* scalar, copy to rgszLex and continue */
+        strcpy (rgszLex[i], szLex);
+      }
+      else { /* array */
+
+        if (2 * (iUB - iLB) != iDoseArg)
+          ReportError (pibIn, RE_TOOMANYPVARS | RE_FATAL, "GetNDoses", NULL);
+
+        for (j = iLB; j < iUB; j++) {
+          sprintf (szTmp, "%s_%ld", szLex, j); /* create names */
+
+          if ((bErr = !(hvar = GetVarHandle (szTmp))))
+            ReportError (pibIn, RE_UNDEFINED | RE_FATAL, szTmp, NULL);
+          else {
+            strcpy (rgszLex[i+j-iLB], szTmp);
+          }
+        } /* for j */
+
+        /* get the final parenthesis */
+       if ((bErr = EGetPunct (pibIn, szTmp, CH_RPAREN)))
+         goto Exit_GetNDoses;
+
+        break; /* get out of the for i loop */
+
+      } /* end else */
+
+  } /* for i */
+
+  if ((bErr = (szTmp[0] != CH_RPAREN)))
     goto Exit_GetNDoses;
 
-  /* Try to get each parm to show all errors */
+  /* We have read in the list and stored it in rgszLex.
+     Try to get each parm value or handle to show all errors */
 
   bOK = TRUE;
 
