@@ -2,7 +2,7 @@
 
    Originally written by Don Robert Maszle
 
-   Copyright (c) 1992-2008 Free Software Foundation, Inc.
+   Copyright (c) 1992-2017 Free Software Foundation, Inc.
 
    This file is part of GNU MCSim.
 
@@ -19,14 +19,6 @@
    You should have received a copy of the GNU General Public License
    along with GNU MCSim; if not, see <http://www.gnu.org/licenses/>
 
-   -- Revisions -----
-     Logfile:  %F%
-    Revision:  %I%
-        Date:  %G%
-     Modtime:  %U%
-      Author:  @a
-   -- SCCS  ---------
-
    Alternate string routines header file.  Routines for when NULL
    pointer dereferencing is a no-no.
 
